import json


class Settings:
	def __init__(self):
		self.discr_order = 1
		self.pressure_discr_order = 1

		self.scalar_formulation = "Laplacian"
		self.tensor_formulation = "LinearElasticity"

		self.normalize_mesh = True

		self.n_refs = 0
		self.vismesh_rel_area = 0.00001
		self.nl_solver_rhs_steps = 1



		self.problem = "Franke"


		self.tend = 1
		self.time_steps = 10


		self.quadrature_order = 4

		self.params = {}
		self.export = {}
		self.advanced_options = {}

		self.problem_params = {}


	def set_problem(self, problem):
		self.problem = problem.name()
		self.problem_params = problem.params()


	def set_material_params(self, name, value):
		self.params[name] = value


	def set_vtu_export_path(self, path, bounda_only=False):
		self.export["vis_mesh"] = path
		self.export["vis_boundary_only"] = bounda_only


	def set_wireframe_export_path(self, path):
		self.export["wire_mesh"] = path


	def set_isolines_export_path(self, path):
		self.export["iso_mesh"] = path


	def set_solution_export_path(self, path):
		self.export["solution"] = path


	def set_advanced_option(self, key, value):
		self.advanced_options[key] = value


	def serialize(self):
		return json.dumps(self, default=lambda o: o.__dict__, sort_keys=True, indent=4)



import unittest

import polyfempy as pf

import os


class BendingTest(unittest.TestCase):
	def test_run(self):
		dir_path = os.path.dirname(os.path.realpath(__file__))
		mesh_path = os.path.join(dir_path, "../3rdparty/data/square_beam.mesh")
		tag_path = os.path.join(dir_path, "../3rdparty/data/square_beam.txt")

		settings = pf.Settings()
		settings.discr_order = 1
		settings.normalize_mesh = False



		settings.set_material_params("E", 200000)
		settings.set_material_params("nu", 0.35)


		settings.tensor_formulation = pf.TensorFormulations.LinearElasticity

		problem = pf.GenericTensor()
		problem.add_dirichlet_value(2, [0, 0, 0])
		problem.add_neumann_value(1,[0, -100, 0])


		settings.vismesh_rel_area = 0.00001

		settings.set_problem(problem)


		solver = pf.Solver()

		solver.settings(settings.serialize())
		solver.load_mesh(mesh_path, tag_path)

		solver.solve()

		solver.export_vtu("bending.vtu")


if __name__ == '__main__':
	unittest.main()



import unittest

import polyfempy as pf

import os


class TorsionTest(unittest.TestCase):
	def test_run(self):
		dir_path = os.path.dirname(os.path.realpath(__file__))
		mesh_path = os.path.join(dir_path, "../3rdparty/data/square_beam_h.HYBRID")

		settings = pf.Settings()
		settings.discr_order = 1
		settings.normalize_mesh = False



		settings.set_material_params("E", 200)
		settings.set_material_params("nu", 0.35)

		settings.nl_solver_rhs_steps = 5
		settings.tensor_formulation = pf.TensorFormulations.NeoHookean

		problem = pf.Torsion()
		problem.fixed_boundary = 5
		problem.turning_boundary = 6
		problem.axis_coordiante = 2
		problem.n_turns = 0.5


		settings.vismesh_rel_area = 0.00001

		settings.set_problem(problem)


		solver = pf.Solver()


		solver.settings(settings.serialize())
		solver.load_mesh(mesh_path)

		solver.solve()

		solver.export_vtu("torsion.vtu")


if __name__ == '__main__':
	unittest.main()





#################################################
############Scalar###############################
class Franke:
	def name(self):
		return "Franke"

	def params(self):
		return {}


class GenericScalar:
	def __init__(self):
		self.rhs = 0
		self.dirichlet_boundary = []
		self.neumann_boundary = []

	def add_dirichlet_value(self, id, value):
		tmp = {}
		tmp["id"] = id
		tmp["value"] = value
		self.dirichlet_boundary.append(tmp)

	def add_neumann_value(self, id, value):
		tmp = {}
		tmp["id"] = id
		tmp["value"] = value
		self.neumann_boundary.append(tmp)

	def name(self):
		return "GenericScalar"

	def params(self):
		return self.__dict__


#################################################
############Elasticity###########################
class Gravity:
	def name(self):
		return "Gravity"

	def params(self):
		return {}


class Torsion:
	def __init__(self):
		self.axis_coordiante = 2
		self.n_turns = 0.5
		self.fixed_boundary = 5
		self.turning_boundary = 6


	def name(self):
		return "TorsionElastic"


	def params(self):
		return self.__dict__


class GenericTensor:
	def __init__(self):
		self.rhs = [0, 0, 0]
		self.dirichlet_boundary = []
		self.neumann_boundary = []

	def add_dirichlet_value(self, id, value, is_dirichlet_dim=None):
		assert(len(value) == 3)
		tmp = {}
		tmp["id"] = id
		tmp["value"] = value
		if is_dirichlet_dim is not None:
			assert(len(is_dirichlet_dim) == 3)
			tmp["dimension"] = is_dirichlet_dim

		self.dirichlet_boundary.append(tmp)

	def add_neumann_value(self, id, value):
		tmp = {}
		tmp["id"] = id
		tmp["value"] = value
		self.neumann_boundary.append(tmp)

	def name(self):
		return "GenericTensor"

	def params(self):
		return self.__dict__


#################################################
############Stokes###############################
class Flow:
	def __init__(self):
		self.inflow = 1
		self.outflow = 3
		self.inflow_amout = 0.25
		self.outflow_amout = 0.25
		self.obstacle = [7]

	def name(self):
		return "Flow"

	def params(self):
		return self.__dict__


class DrivenCavity:
	def name(self):
		return "DrivenCavity"

	def params(self):
		return {}
